@@ -4,19 +4,19 @@ import io
 import os
 import os.path
 import shutil
+import struct
 import sys
 import tempfile
 import warnings
 
 from pytest import mark, raises
 
-from wand.image import ClosedImageError, Image
+from wand.image import ClosedImageError, Image, IMAGE_LAYER_METHOD
 from wand.color import Color
 from wand.compat import PY3, string_type, text, text_type
-from wand.exceptions import MissingDelegateError
+from wand.exceptions import OptionError, MissingDelegateError
 from wand.font import Font
 
-
 try:
     filesystem_encoding = sys.getfilesystemencoding()
 except RuntimeError:
@@ -36,14 +36,22 @@ else:
 
 def test_empty_image():
     with Image() as img:
-        assert img.size == (0,0)
+        assert img.size == (0, 0)
+        assert repr(img) == '<wand.image.Image: (empty)>'
+
+
+def test_image_invalid_params():
+    with raises(TypeError):
+        Image(image=Image(), width=100, height=100)
+    with raises(TypeError):
+        Image(image=Image(), blob=b"blob")
+    with raises(TypeError):
+        Image(image=b"blob")
 
 
 def test_blank_image():
     gray = Color('#ccc')
     transparent = Color('transparent')
-    with raises(TypeError):
-        Image(height=0, filename='/test.png')
     with raises(TypeError):
         Image(width=0, height=0)
     with Image(width=20, height=10) as img:
@@ -53,14 +61,36 @@ def test_blank_image():
         assert img[10, 5] == gray
 
 
+def test_raw_image(fx_asset):
+    b = b"".join([struct.pack("BBB", i, j, 0)
+                  for i in range(256) for j in range(256)])
+    with raises(ValueError):
+        Image(blob=b, depth=6)
+    with raises(TypeError):
+        Image(blob=b, depth=8, width=0, height=0, format="RGB")
+    with raises(TypeError):
+        Image(blob=b, depth=8, width=256, height=256, format=1)
+    with Image(blob=b, depth=8, width=256, height=256, format="RGB") as img:
+        assert img.size == (256, 256)
+        assert img[0, 0] == Color('#000000')
+        assert img[255, 255] == Color('#ffff00')
+        assert img[64, 128] == Color('#804000')
+    with Image(filename=str(fx_asset.join('blob.rgb')),
+               depth=8, width=256, height=256, format="RGB") as img:
+        assert img.size == (256, 256)
+        assert img[0, 0] == Color('#000000')
+        assert img[255, 255] == Color('#ffff00')
+        assert img[64, 128] == Color('#804000')
+
+
 def test_clear_image(fx_asset):
     with Image() as img:
         img.read(filename=str(fx_asset.join('mona-lisa.jpg')))
-        assert img.size == (402,599)
+        assert img.size == (402, 599)
         img.clear()
-        assert img.size == (0,0)
+        assert img.size == (0, 0)
         img.read(filename=str(fx_asset.join('beach.jpg')))
-        assert img.size == (800,600)
+        assert img.size == (800, 600)
 
 
 def test_read_from_filename(fx_asset):
@@ -209,18 +239,18 @@ def test_save_to_file(fx_asset):
         with Image(file=buffer) as saved:
             assert saved.size == (402, 599)
     buffer.close()
-    
-    
+
+
 def test_save_full_animated_gif_to_file(fx_asset):
     """Save all frames of an animated to a Python file object."""
     temp_filename = os.path.join(tempfile.mkdtemp(), 'savetest.gif')
     orig_filename = str(fx_asset.join('nocomments.gif'))
     with open(temp_filename, 'w+b') as fp:
-        with Image(filename= orig_filename) as orig:
-            orig.save(file= fp)
+        with Image(filename=orig_filename) as orig:
+            orig.save(file=fp)
     assert os.path.isfile(temp_filename)
-    with Image(filename= orig_filename) as orig:
-        with Image(filename= temp_filename) as temp:
+    with Image(filename=orig_filename) as orig:
+        with Image(filename=temp_filename) as temp:
             assert len(orig.sequence) == len(temp.sequence)
     os.remove(temp_filename)
 
@@ -246,7 +276,8 @@ def test_make_blob(fx_asset):
             img.make_blob(123)
     svg = b'''
     <svg width="100px" height="100px">
-    <circle cx="100" cy="50" r="40" stroke="black" stroke-width="2" fill="red"/>
+        <circle cx="100" cy="50" r="40" stroke="black"
+         stroke-width="2" fill="red" />
     </svg>
     '''
     with Image(blob=svg, format='svg') as img:
@@ -293,7 +324,7 @@ def test_set_resolution_02(fx_asset):
 def test_set_resolution_03(fx_asset):
     """Sets image resolution on constructor"""
     with Image(filename=str(fx_asset.join('sample.pdf')),
-               resolution=(100,100)) as img:
+               resolution=(100, 100)) as img:
         assert img.resolution == (100, 100)
 
 
@@ -315,10 +346,25 @@ def test_get_units(fx_asset):
 def test_set_units(fx_asset):
     """Sets the image resolution units."""
     with Image(filename=str(fx_asset.join('watermark.png'))) as img:
-        img.units="pixelspercentimeter"
+        img.units = "pixelspercentimeter"
         assert img.units == "pixelspercentimeter"
 
 
+def test_get_virtual_pixel(fx_asset):
+    """Gets image virtual pixel"""
+    with Image(filename=str(fx_asset.join('mona-lisa.jpg'))) as img:
+        assert img.virtual_pixel == "undefined"
+
+
+def test_set_virtual_pixel(fx_asset):
+    """Sets image virtual pixel"""
+    with Image(filename=str(fx_asset.join('mona-lisa.jpg'))) as img:
+        img.virtual_pixel = "tile"
+        assert img.virtual_pixel == "tile"
+        with raises(ValueError):
+            img.virtual_pixel = "nothing"
+
+
 def test_get_colorspace(fx_asset):
     """Gets the image colorspace"""
     with Image(filename=str(fx_asset.join('mona-lisa.jpg'))) as img:
@@ -482,11 +528,11 @@ def test_iterate(fx_asset):
                 for i, row in enumerate(img):
                     assert len(row) == 300
                     if i % 3:
-                        continue # avoid slowness
+                        continue  # avoid slowness
                     if 100 <= i < 200:
                         for x, color in enumerate(row):
                             if x % 3:
-                                continue # avoid slowness
+                                continue  # avoid slowness
                             if 100 <= x < 200:
                                 assert color == black
                             else:
@@ -500,7 +546,7 @@ def test_iterate(fx_asset):
 def test_slice_clone(fx_asset):
     """Clones using slicing."""
     with Image(filename=str(fx_asset.join('mona-lisa.jpg'))) as img:
-        with img[:,:] as cloned:
+        with img[:, :] as cloned:
             assert img.size == cloned.size
 
 
@@ -663,6 +709,53 @@ def test_crop_error(fx_asset):
             img.crop(bottom=1, height=2)
 
 
+def test_crop_gravity(fx_asset):
+    with Image(filename=str(fx_asset.join('croptest.png'))) as img:
+        width = int(img.width / 3)
+        height = int(img.height / 3)
+        mid_width = int(width / 2)
+        mid_height = int(height / 2)
+        with img.clone() as center:
+            center.crop(width=width, height=height, gravity='center')
+            assert center[mid_width, mid_height] == Color('black')
+        with img.clone() as northwest:
+            northwest.crop(width=width, height=height, gravity='north_west')
+            assert northwest[mid_width, mid_height] == Color('transparent')
+        with img.clone() as southeast:
+            southeast.crop(width=width, height=height, gravity='south_east')
+            assert southeast[mid_width, mid_height] == Color('transparent')
+
+
+def test_crop_gravity_error(fx_asset):
+    with Image(filename=str(fx_asset.join('croptest.png'))) as img:
+        with raises(TypeError):
+            img.crop(gravity='center')
+        with raises(ValueError):
+            img.crop(width=1, height=1, gravity='nowhere')
+
+
+@mark.slow
+def test_distort(fx_asset):
+    """Distort image."""
+    with Image(filename=str(fx_asset.join('mona-lisa.jpg'))) as img:
+        with Color('skyblue') as color:
+            img.matte_color = color
+            img.virtual_pixel = 'tile'
+            img.distort('perspective', (0, 0, 20, 60, 90, 0,
+                                        70, 63, 0, 90, 5, 83,
+                                        90, 90, 85, 88))
+            assert img[img.width - 1, 0] == color
+
+
+def test_distort_error(fx_asset):
+    """Distort image with user error"""
+    with Image(filename=str(fx_asset.join('mona-lisa.jpg'))) as img:
+        with raises(ValueError):
+            img.distort('mirror', (1,))
+        with raises(TypeError):
+            img.distort('perspective', 1)
+
+
 @mark.parametrize(('method'), [
     ('resize'),
     ('sample'),
@@ -917,17 +1010,22 @@ def test_object_hash(fx_asset):
 def test_get_alpha_channel(fx_asset):
     """Checks if image has alpha channel."""
     with Image(filename=str(fx_asset.join('watermark.png'))) as img:
-        assert img.alpha_channel == True
+        assert img.alpha_channel is True
     with Image(filename=str(fx_asset.join('mona-lisa.jpg'))) as img:
-        assert img.alpha_channel == False
+        assert img.alpha_channel is False
 
 
 def test_set_alpha_channel(fx_asset):
     """Sets alpha channel to off."""
     with Image(filename=str(fx_asset.join('watermark.png'))) as img:
-        assert img.alpha_channel == True
+        img.alpha_channel = 'on'
+        assert img.alpha_channel is True
         img.alpha_channel = False
-        assert img.alpha_channel == False
+        assert img.alpha_channel is False
+        img.alpha_channel = 'opaque'
+        assert img[0, 0].alpha == 1.0
+        with raises(ValueError):
+            img.alpha_channel = 'watermark'
 
 
 def test_get_background_color(fx_asset):
@@ -944,6 +1042,15 @@ def test_set_background_color(fx_asset):
             assert img.background_color == color
 
 
+def test_set_get_matte_color(fx_asset):
+    with Image(filename='rose:') as img:
+        with Color('navy') as color:
+            img.matte_color = color
+            assert img.matte_color == color
+            with raises(TypeError):
+                img.matte_color = False
+
+
 def test_transparentize(fx_asset):
     with Image(filename=str(fx_asset.join('croptest.png'))) as im:
         with Color('transparent') as transparent:
@@ -1033,7 +1140,7 @@ def test_composite(fx_asset):
             assert img[0, 0] == orig[0, 0]
             assert img[0, img.height - 1] == orig[0, orig.height - 1]
             assert img[img.width - 1, 0] == orig[orig.width - 1, 0]
-            assert (img[img.width - 1, img.height - 1] == 
+            assert (img[img.width - 1, img.height - 1] ==
                     orig[orig.width - 1, img.height - 1])
             # These pixels should be the almost black:
             assert img[70, 100].red <= 1
@@ -1077,6 +1184,14 @@ def test_composite_channel(fx_asset):
                         assert ic.blue == oc.blue
 
 
+def test_compare(fx_asset):
+    with Image(filename=str(fx_asset.join('beach.jpg'))) as orig:
+        with Image(filename=str(fx_asset.join('watermark_beach.jpg'))) as img:
+            cmp_img, err = orig.compare(img, 'absolute')
+            cmp_img, err = orig.compare(img, 'mean_absolute')
+            cmp_img, err = orig.compare(img, 'root_mean_square')
+
+
 def test_liquid_rescale(fx_asset):
     def assert_equal_except_alpha(a, b):
         with a:
@@ -1131,6 +1246,17 @@ def test_caption(fx_asset):
         )
 
 
+def test_caption_without_font(fx_asset):
+    with Image(width=144, height=192, background=Color('#1e50a2')) as img:
+        with raises(TypeError):
+            img.caption(
+                'Test message',
+                left=5, top=144,
+                width=134, height=20,
+                gravity='center'
+            )
+
+
 def test_setfont(fx_asset):
     with Image(width=144, height=192, background=Color('#1e50a2')) as img:
         font = Font(
@@ -1155,9 +1281,9 @@ def test_setgravity():
 
 def test_negate_default(display, fx_asset):
     def test(c1, c2):
-        assert c1.red_int8 + c2.red_int8 == 255 and \
-               c1.green_int8 + c2.green_int8 == 255 and \
-               c1.blue_int8 + c2.blue_int8 == 255
+        assert (c1.red_int8 + c2.red_int8 == 255 and
+                c1.green_int8 + c2.green_int8 == 255 and
+                c1.blue_int8 + c2.blue_int8 == 255)
     with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
         display(img)
         left_top = img[0, 0]
@@ -1171,6 +1297,114 @@ def test_negate_default(display, fx_asset):
         test(right_bottom, img[-1, -1])
 
 
+def test_threshold(fx_asset):
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        top = int(img.height * 0.25)
+        btm = int(img.height * 0.75)
+        print(img[0, top], img[0, btm])
+        img.threshold(0.5)
+        print(img[0, top], img[0, btm])
+        with img[0, top] as white:
+            assert white.red_int8 == white.green_int8 == white.blue_int8 == 255
+        with img[0, btm] as black:
+            assert black.red_int8 == black.green_int8 == black.blue_int8 == 0
+
+
+def test_threshold_channel(fx_asset):
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        top = int(img.height * 0.25)
+        btm = int(img.height * 0.75)
+        print(img[0, top], img[0, btm])
+        img.threshold(0.0, 'red')
+        img.threshold(0.5, 'green')
+        img.threshold(1.0, 'blue')
+        print(img[0, top], img[0, btm])
+        # The top half of the image should be yellow, and the bottom half red.
+        with img[0, top] as yellow:
+            assert (yellow.red_int8 == yellow.green_int8 == 255 and
+                    yellow.blue_int8 == 0)
+        with img[0, btm] as red:
+            assert (red.red_int8 == 255 and
+                    red.green_int8 == red.blue_int8 == 0)
+
+
+def test_contrast_stretch(fx_asset):
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        img.contrast_stretch(0.15)
+        with img[0, 10] as left_top:
+            assert left_top.red_int8 == 255
+        with img[0, 90] as left_bottom:
+            assert left_bottom.red_int8 == 0
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        img.contrast_stretch(0.15, channel='red')
+        with img[0, 10] as left_top:
+            assert left_top.red_int8 == 255
+        with img[0, 90] as left_bottom:
+            assert left_bottom.red_int8 == 0
+
+
+def test_contrast_stretch_user_error(fx_asset):
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        with raises(TypeError):
+            img.contrast_stretch('NaN')
+        with raises(TypeError):
+            img.contrast_stretch(0.1, 'NaN')
+        with raises(ValueError):
+            img.contrast_stretch(0.1, channel='Not a channel')
+
+
+def test_gamma(fx_asset):
+    # Value under 1.0 is darker, and above 1.0 is lighter
+    middle_point = 75, 50
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        with img.clone() as lighter:
+            lighter.gamma(1.5)
+            assert img[middle_point].red < lighter[middle_point].red
+        with img.clone() as darker:
+            darker.gamma(0.5)
+            assert img[middle_point].red > darker[middle_point].red
+
+
+def test_gamma_channel(fx_asset):
+    # Value under 1.0 is darker, and above 1.0 is lighter
+    middle_point = 75, 50
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        with img.clone() as lighter:
+            lighter.gamma(1.5, channel='red')
+            assert img[middle_point].red < lighter[middle_point].red
+        with img.clone() as darker:
+            darker.gamma(0.5, channel='red')
+            assert img[middle_point].red > darker[middle_point].red
+
+
+def test_gamma_user_error(fx_asset):
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        with raises(TypeError):
+            img.gamma('NaN;')
+        with raises(ValueError):
+            img.gamma(0.0, 'no channel')
+
+
+def test_linear_stretch(fx_asset):
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        img.linear_stretch(black_point=0.15,
+                           white_point=0.15)
+        with img[0, 10] as left_top:
+            assert left_top.red_int8 == 255
+        with img[0, 90] as left_bottom:
+            assert left_bottom.red_int8 == 0
+
+
+def test_linear_stretch_user_error(fx_asset):
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        with raises(TypeError):
+            img.linear_stretch(white_point='NaN',
+                               black_point=0.5)
+        with raises(TypeError):
+            img.linear_stretch(white_point=0.5,
+                               black_point='NaN')
+
+
 def test_normalize_default(display, fx_asset):
     with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
         display(img)
@@ -1181,7 +1415,7 @@ def test_normalize_default(display, fx_asset):
         print(left_top, left_bottom, right_top, right_bottom)
         img.normalize()
         display(img)
-        print(img[0,0], img[0,-1], img[-1,0], img[-1,-1])
+        print(img[0, 0], img[0, -1], img[-1, 0], img[-1, -1])
         assert img[0, 0] != left_top
         assert img[0, -1] != left_bottom
         assert img[-1, 0] != right_top
@@ -1189,8 +1423,8 @@ def test_normalize_default(display, fx_asset):
         with img[0, 0] as left_top:
             assert left_top.red == left_top.green == left_top.blue == 1
         with img[-1, -1] as right_bottom:
-            assert (right_bottom.red == right_bottom.green
-                                     == right_bottom.blue == 0)
+            assert (right_bottom.red == right_bottom.green ==
+                    right_bottom.blue == 0)
 
 
 def test_normalize_channel(fx_asset):
@@ -1217,6 +1451,111 @@ def test_normalize_channel(fx_asset):
             assert getattr(img[-1, -1], c) == getattr(right_bottom, c)
 
 
+def test_level_default(fx_asset):
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        # Adjust the levels to make this image entirely black
+        img.level(1, 1)
+        with img[0, 0] as dark:
+            assert dark.red_int8 <= dark.green_int8 <= dark.blue_int8 <= 0
+        with img[0, -1] as dark:
+            assert dark.red_int8 <= dark.green_int8 <= dark.blue_int8 <= 0
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        # Adjust the levels to make this image entirely white
+        img.level(0, 0)
+        with img[0, 0] as light:
+            assert light.red_int8 >= light.green_int8 >= light.blue_int8 >= 255
+        with img[0, -1] as light:
+            assert light.red_int8 >= light.green_int8 >= light.blue_int8 >= 255
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        # Adjust the image's gamma to darken its midtones
+        img.level(gamma=0.5)
+        with img[0, len(img) // 2] as light:
+            assert light.red_int8 <= light.green_int8 <= light.blue_int8 <= 65
+            assert light.red_int8 >= light.green_int8 >= light.blue_int8 >= 60
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        # Adjust the image's gamma to lighten its midtones
+        img.level(0, 1, 2.5)
+        with img[0, len(img) // 2] as light:
+            assert light.red_int8 <= light.green_int8 <= light.blue_int8 <= 195
+            assert light.red_int8 >= light.green_int8 >= light.blue_int8 >= 190
+
+
+def test_level_channel(fx_asset):
+    for chan in ('red', 'green', 'blue'):
+        c = chan + '_int8'
+        with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+            # Adjust each channel level to make it entirely black
+            img.level(1, 1, channel=chan)
+            assert(getattr(img[0, 0], c) <= 0)
+            assert(getattr(img[0, -1], c) <= 0)
+        with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+            # Adjust each channel level to make it entirely white
+            img.level(0, 0, channel=chan)
+            assert(getattr(img[0, 0], c) >= 255)
+            assert(getattr(img[0, -1], c) >= 255)
+        with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+            # Adjust each channel's gamma to darken its midtones
+            img.level(gamma=0.5, channel=chan)
+            with img[0, len(img) // 2] as light:
+                assert(getattr(light, c) <= 65)
+                assert(getattr(light, c) >= 60)
+        with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+            # Adjust each channel's gamma to lighten its midtones
+            img.level(0, 1, 2.5, chan)
+            with img[0, len(img) // 2] as light:
+                assert(getattr(light, c) >= 190)
+                assert(getattr(light, c) <= 195)
+
+
+def test_level_user_error(fx_asset):
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        with raises(TypeError):
+            img.level(black='NaN')
+        with raises(TypeError):
+            img.level(white='NaN')
+        with raises(TypeError):
+            img.level(gamma='NaN')
+        with raises(ValueError):
+            img.level(channel='404')
+
+
+def test_equalize(fx_asset):
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        print(img[0, 0], img[0, -1])
+        img.equalize()
+        print(img[0, 0], img[0, -1])
+        # The top row should be nearly white, and the bottom nearly black.
+        with img[0, 0] as light:
+            assert light.red_int8 >= light.green_int8 >= light.blue_int8 >= 250
+        with img[0, -1] as dark:
+            assert dark.red_int8 <= dark.green_int8 <= dark.blue_int8 <= 5
+
+
+def test_evaluate(fx_asset):
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        with img.clone() as percent_img:
+            fifty_percent = percent_img.quantum_range * 0.5
+            percent_img.evaluate('set', fifty_percent)
+            with percent_img[10, 10] as gray:
+                assert abs(gray.red - Color('gray50').red) < 0.01
+        with img.clone() as literal_img:
+            literal_img.evaluate('divide', 2, channel='red')
+            with img[0, 0] as org_color:
+                expected_color = (org_color.red_int8 * 0.5)
+                with literal_img[0, 0] as actual_color:
+                    assert abs(expected_color - actual_color.red_int8) < 1
+
+
+def test_evaluate_user_error(fx_asset):
+    with Image(filename=str(fx_asset.join('gray_range.jpg'))) as img:
+        with raises(ValueError):
+            img.evaluate(operator='Nothing')
+        with raises(TypeError):
+            img.evaluate(operator='set', value='NaN')
+        with raises(ValueError):
+            img.evaluate(operator='set', value=1.0, channel='Not a channel')
+
+
 def test_flip(fx_asset):
     with Image(filename=str(fx_asset.join('beach.jpg'))) as img:
         with img.clone() as flipped:
@@ -1237,6 +1576,95 @@ def test_flop(fx_asset):
             assert flopped[-1, -1] == img[0, -1]
 
 
+def test_frame(fx_asset):
+    with Image(filename=str(fx_asset.join('mona-lisa.jpg'))) as img:
+        img.frame(width=4, height=4)
+        assert img[0, 0] == img[-1, -1]
+        assert img[-1, 0] == img[0, -1]
+    with Color('green') as green:
+        with Image(filename=str(fx_asset.join('mona-lisa.jpg'))) as img:
+            img.frame(matte=green, width=2, height=2)
+            assert img[0, 0] == green
+            assert img[-1, -1] == green
+
+
+def test_frame_error(fx_asset):
+    with Image(filename=str(fx_asset.join('mona-lisa.jpg'))) as img:
+        with raises(TypeError):
+            img.frame(width='one')
+        with raises(TypeError):
+            img.frame(height=3.5)
+        with raises(TypeError):
+            img.frame(matte='green')
+        with raises(TypeError):
+            img.frame(inner_bevel=None)
+        with raises(TypeError):
+            img.frame(outer_bevel='large')
+
+
+def test_function(fx_asset):
+    with Image(filename=str(fx_asset.join('croptest.png'))) as img:
+        img.function(function='polynomial',
+                     arguments=(4, -4, 1))
+        assert img[150, 150] == Color('white')
+        img.function(function='sinusoid',
+                     arguments=(1,),
+                     channel='red')
+        assert abs(img[150, 150].red - Color('#80FFFF').red) < 0.01
+
+
+def test_function_error(fx_asset):
+    with Image(filename=str(fx_asset.join('croptest.png'))) as img:
+        with raises(ValueError):
+            img.function('bad function', 1)
+        with raises(TypeError):
+            img.function('sinusoid', 1)
+        with raises(ValueError):
+            img.function('sinusoid', (1,), channel='bad channel')
+
+
+def test_fx(fx_asset):
+    with Image(width=2, height=2, background=Color('black')) as xc1:
+        # NavyBlue == #000080
+        with xc1.fx('0.5019', channel='blue') as xc2:
+            assert abs(xc2[0, 0].blue - Color('navy').blue) < 0.0001
+
+    with Image(width=2, height=1, background=Color('white')) as xc1:
+        with xc1.fx('0') as xc2:
+            assert xc2[0, 0].red == 0
+
+
+def test_fx_error(fx_asset):
+    with Image() as empty_wand:
+        with raises(AttributeError):
+            with empty_wand.fx('8'):
+                pass
+    with Image(filename='rose:') as xc:
+        with raises(OptionError):
+            with xc.fx('/0'):
+                pass
+        with raises(TypeError):
+            with xc.fx(('p[0,0]',)):
+                pass
+        with raises(ValueError):
+            with xc.fx('p[0,0]', True):
+                pass
+
+
+def test_transpose(fx_asset):
+    with Image(filename=str(fx_asset.join('beach.jpg'))) as img:
+        with img.clone() as transposed:
+            transposed.transpose()
+            assert transposed[501, 501] == Color('srgb(205,196,179)')
+
+
+def test_transverse(fx_asset):
+    with Image(filename=str(fx_asset.join('beach.jpg'))) as img:
+        with img.clone() as transversed:
+            transversed.transverse()
+            assert transversed[500, 500] == Color('srgb(96,136,185)')
+
+
 def test_get_orientation(fx_asset):
     with Image(filename=str(fx_asset.join('sasha.jpg'))) as img:
         assert img.orientation == 'undefined'
@@ -1251,6 +1679,31 @@ def test_set_orientation(fx_asset):
         assert img.orientation == 'bottom_right'
 
 
+def test_auto_orientation(fx_asset):
+    with Image(filename=str(fx_asset.join('beach.jpg'))) as img:
+            # if orientation is undefined nothing should be changed
+            before = img[100, 100]
+            img.auto_orient()
+            after = img[100, 100]
+            assert before == after
+            assert img.orientation == 'top_left'
+
+    with Image(filename=str(fx_asset.join('orientationtest.jpg'))) as original:
+        with original.clone() as img:
+            # now we should get a flipped image
+            assert img.orientation == 'bottom_left'
+            before = img[100, 100]
+            img.auto_orient()
+            after = img[100, 100]
+            assert before != after
+            assert img.orientation == 'top_left'
+
+            assert img[0, 0] == original[0, -1]
+            assert img[0, -1] == original[0, 0]
+            assert img[-1, 0] == original[-1, -1]
+            assert img[-1, -1] == original[-1, 0]
+
+
 def test_histogram(fx_asset):
     with Image(filename=str(fx_asset.join('trim-color-test.png'))) as a:
         h = a.histogram
@@ -1283,9 +1736,9 @@ def test_gaussian_blur(fx_asset, display):
 
 def test_modulate(fx_asset, display):
     with Image(filename=str(fx_asset.join('sasha.jpg'))) as img:
-        before = img[100,100]
+        before = img[100, 100]
         img.modulate(120, 120, 120)
-        after = img[100,100]
+        after = img[100, 100]
         assert before != after
         assert 0.98 <= after.red <= 0.99
         assert 0.98 <= after.green <= 0.99
@@ -1318,3 +1771,182 @@ def test_issue_150(fx_asset, tmpdir):
         img.format = 'pjpeg'
         with open(str(tmpdir.join('out.jpg')), 'wb') as f:
             img.save(file=f)
+
+
+@mark.slow
+def test_quantize(fx_asset):
+    number_colors = 64
+    with Image(filename=str(fx_asset.join('mona-lisa.jpg'))) as img:
+        colors = set([color for row in img for color in row])
+        assert len(colors) > number_colors
+
+    with Image(filename=str(fx_asset.join('mona-lisa.jpg'))) as img:
+        with raises(TypeError):
+            img.quantize(str(number_colors), 'undefined', 0, True, True)
+
+        with raises(TypeError):
+            img.quantize(number_colors, 0, 0, True, True)
+
+        with raises(TypeError):
+            img.quantize(number_colors, 'undefined', 'depth', True, True)
+
+        with raises(TypeError):
+            img.quantize(number_colors, 'undefined', 0, 1, True)
+
+        with raises(TypeError):
+            img.quantize(number_colors, 'undefined', 0, True, 1)
+
+        img.quantize(number_colors, 'undefined', 0, True, True)
+        colors = set([color for row in img for color in row])
+        assert colors
+        assert len(colors) <= number_colors
+
+
+def test_transform_colorspace(fx_asset):
+    with Image(filename=str(fx_asset.join('cmyk.jpg'))) as img:
+        with raises(TypeError):
+            img.transform_colorspace('unknown')
+
+        img.transform_colorspace('srgb')
+        assert img.colorspace == 'srgb'
+
+
+def test_merge_layers_basic(fx_asset):
+    for method in ['merge', 'flatten', 'mosaic']:
+            with Image(filename=str(fx_asset.join('cmyk.jpg'))) as img1:
+                orig_size = img1.size
+                with Image(filename=str(fx_asset.join('cmyk.jpg'))) as img2:
+                    img1.sequence.append(img2)
+                    assert len(img1.sequence) == 2
+                    img1.merge_layers(method)
+                    assert len(img1.sequence) == 1
+                    assert img1.size == orig_size
+
+
+def test_merge_layers_bad_method(fx_asset):
+    with Image(filename=str(fx_asset.join('cmyk.jpg'))) as img:
+        for method in IMAGE_LAYER_METHOD + ('', 'junk'):
+            if method in ['merge', 'flatten', 'mosaic']:
+                continue  # skip the valid ones
+            with raises(TypeError):
+                img.merge_layers(method)
+
+
+def test_merge_layers_method_merge(fx_asset):
+    with Image(width=16, height=16) as img1:
+        img1.background_color = Color('black')
+        img1.alpha_channel = False
+        with Image(width=32, height=32) as img2:
+            img2.background_color = Color('white')
+            img2.alpha_channel = False
+            img2.transform(crop='16x16+8+8')
+
+            img1.sequence.append(img2)
+            img1.merge_layers('merge')
+            assert img1.size == (24, 24)
+
+
+def test_merge_layers_method_merge_neg_offset(fx_asset):
+    with Image(width=16, height=16) as img1:
+        img1.background_color = Color('black')
+        img1.alpha_channel = False
+        with Image(width=16, height=16) as img2:
+            img2.background_color = Color('white')
+            img2.alpha_channel = False
+            img2.page = (16, 16, -8, -8)
+
+            img1.sequence.append(img2)
+            img1.merge_layers('merge')
+            assert img1.size == (24, 24)
+
+
+def test_merge_layers_method_flatten(fx_asset):
+    with Image(width=16, height=16) as img1:
+        img1.background_color = Color('black')
+        img1.alpha_channel = False
+        with Image(width=32, height=32) as img2:
+            img2.background_color = Color('white')
+            img2.alpha_channel = False
+            img2.transform(crop='16x16+8+8')
+
+            img1.sequence.append(img2)
+            img1.merge_layers('flatten')
+            assert img1.size == (16, 16)
+
+
+def test_merge_layers_method_mosaic(fx_asset):
+    with Image(width=16, height=16) as img1:
+        img1.background_color = Color('black')
+        img1.alpha_channel = False
+        with Image(width=32, height=32) as img2:
+            img2.background_color = Color('white')
+            img2.alpha_channel = False
+            img2.transform(crop='16x16+8+8')
+
+            img1.sequence.append(img2)
+            img1.merge_layers('mosaic')
+            assert img1.size == (24, 24)
+
+
+def test_merge_layers_method_mosaic_neg_offset(fx_asset):
+    with Image(width=16, height=16) as img1:
+        img1.background_color = Color('black')
+        img1.alpha_channel = False
+        with Image(width=16, height=16) as img2:
+            img2.background_color = Color('white')
+            img2.alpha_channel = False
+            img2.page = (16, 16, -8, -8)
+
+            img1.sequence.append(img2)
+            img1.merge_layers('mosaic')
+            assert img1.size == (16, 16)
+
+
+def test_page_basic(fx_asset):
+    with Image(filename=str(fx_asset.join('watermark.png'))) as img1:
+        assert img1.page == (640, 480, 0, 0)
+        assert img1.page_width == 640
+        assert img1.page_height == 480
+        assert img1.page_x == 0
+        assert img1.page_y == 0
+
+
+def test_page_offset(fx_asset):
+    with Image(filename=str(fx_asset.join('watermark-offset.png'))) as img1:
+        assert img1.page == (640, 480, 12, 13)
+        assert img1.page_width == 640
+        assert img1.page_height == 480
+        assert img1.page_x == 12
+        assert img1.page_y == 13
+
+
+def test_page_setter(fx_asset):
+    with Image(filename=str(fx_asset.join('watermark.png'))) as img1:
+        assert img1.page == (640, 480, 0, 0)
+        img1.page = (640, 480, 0, 0)
+        assert img1.page == (640, 480, 0, 0)
+        img1.page = (640, 480, 12, 13)
+        assert img1.page == (640, 480, 12, 13)
+        img1.page = (640, 480, -12, 13)
+        assert img1.page == (640, 480, -12, 13)
+        img1.page = (640, 480, 12, -13)
+        assert img1.page == (640, 480, 12, -13)
+        img1.page = (6400, 4800, 2, 3)
+        assert img1.page == (6400, 4800, 2, 3)
+
+
+def test_page_setter_items(fx_asset):
+    with Image(filename=str(fx_asset.join('watermark.png'))) as img1:
+        assert img1.page == (640, 480, 0, 0)
+        img1.page_width = 6400
+        assert img1.page == (6400, 480, 0, 0)
+        img1.page_height = 4800
+        assert img1.page == (6400, 4800, 0, 0)
+        img1.page_x = 12
+        assert img1.page == (6400, 4800, 12, 0)
+        img1.page_y = 13
+        assert img1.page == (6400, 4800, 12, 13)
+        img1.page_x = -12
+        assert img1.page == (6400, 4800, -12, 13)
+        img1.page_y = -13
+        assert img1.page == (6400, 4800, -12, -13)
